@@ -4,9 +4,11 @@ extern crate trezor_crypto_sys as sys;
 
 pub extern crate generic_array;
 
+pub mod bip39;
 pub mod curve;
 pub mod ecdsa;
 pub mod ed25519;
 pub mod hasher;
 pub mod hd_node;
 pub mod signature;
+mod utils;
